@@ -8,9 +8,11 @@ from __future__ import absolute_import
 
 import re
 import os.path
+import sys
 from collections import defaultdict
 
 from coverage.plugin import CoveragePlugin, FileTracer, FileReporter  # requires coverage.py 4.0+
+from coverage.files import canonical_filename
 
 from .Utils import find_root_package_dir, is_package_dir, open_source_file
 
@@ -18,14 +20,17 @@ from .Utils import find_root_package_dir, is_package_dir, open_source_file
 from . import __version__
 
 
+C_FILE_EXTENSIONS = ['.c', '.cpp', '.cc', '.cxx']
+MODULE_FILE_EXTENSIONS = set(['.py', '.pyx', '.pxd'] + C_FILE_EXTENSIONS)
+
+
 def _find_c_source(base_path):
-    if os.path.exists(base_path + '.c'):
-        c_file = base_path + '.c'
-    elif os.path.exists(base_path + '.cpp'):
-        c_file = base_path + '.cpp'
-    else:
-        c_file = None
-    return c_file
+    file_exists = os.path.exists
+    for ext in C_FILE_EXTENSIONS:
+        file_name = base_path + ext
+        if file_exists(file_name):
+            return file_name
+    return None
 
 
 def _find_dep_file_path(main_file, file_path):
@@ -35,7 +40,13 @@ def _find_dep_file_path(main_file, file_path):
         pxi_file_path = os.path.join(os.path.dirname(main_file), file_path)
         if os.path.exists(pxi_file_path):
             abs_path = os.path.abspath(pxi_file_path)
-    return abs_path
+    # search sys.path for external locations if a valid file hasn't been found
+    if not os.path.exists(abs_path):
+        for sys_path in sys.path:
+            test_path = os.path.realpath(os.path.join(sys_path, file_path))
+            if os.path.exists(test_path):
+                return canonical_filename(test_path)
+    return canonical_filename(abs_path)
 
 
 class Plugin(CoveragePlugin):
@@ -56,7 +67,7 @@ class Plugin(CoveragePlugin):
         if filename.startswith('<') or filename.startswith('memory:'):
             return None
         c_file = py_file = None
-        filename = os.path.abspath(filename)
+        filename = canonical_filename(os.path.abspath(filename))
         if self._c_files_map and filename in self._c_files_map:
             c_file = self._c_files_map[filename][0]
 
@@ -84,7 +95,7 @@ class Plugin(CoveragePlugin):
         #    from coverage.python import PythonFileReporter
         #    return PythonFileReporter(filename)
 
-        filename = os.path.abspath(filename)
+        filename = canonical_filename(os.path.abspath(filename))
         if self._c_files_map and filename in self._c_files_map:
             c_file, rel_file_path, code = self._c_files_map[filename]
         else:
@@ -92,15 +103,23 @@ class Plugin(CoveragePlugin):
             if not c_file:
                 return None  # unknown file
             rel_file_path, code = self._parse_lines(c_file, filename)
+            if code is None:
+                return None  # no source found
         return CythonModuleReporter(c_file, filename, rel_file_path, code)
 
     def _find_source_files(self, filename):
         basename, ext = os.path.splitext(filename)
         ext = ext.lower()
-        if ext in ('.py', '.pyx', '.pxd', '.c', '.cpp'):
+        if ext in MODULE_FILE_EXTENSIONS:
             pass
-        elif ext in ('.so', '.pyd'):
-            platform_suffix = re.search(r'[.]cpython-[0-9]+[a-z]*$', basename, re.I)
+        elif ext == '.pyd':
+            # Windows extension module
+            platform_suffix = re.search(r'[.]cp[0-9]+-win[_a-z0-9]*$', basename, re.I)
+            if platform_suffix:
+                basename = basename[:platform_suffix.start()]
+        elif ext == '.so':
+            # Linux/Unix/Mac extension module
+            platform_suffix = re.search(r'[.](?:cpython|pypy)-[0-9]+[-_a-z0-9]*$', basename, re.I)
             if platform_suffix:
                 basename = basename[:platform_suffix.start()]
         elif ext == '.pxi':
@@ -114,7 +133,7 @@ class Plugin(CoveragePlugin):
             # none of our business
             return None, None
 
-        c_file = filename if ext in ('.c', '.cpp') else _find_c_source(basename)
+        c_file = filename if ext in C_FILE_EXTENSIONS else _find_c_source(basename)
         if c_file is None:
             # a module "pkg/mod.so" can have a source file "pkg/pkg.mod.c"
             package_root = find_root_package_dir.uncached(filename)
@@ -148,7 +167,7 @@ class Plugin(CoveragePlugin):
         splitext = os.path.splitext
         for filename in os.listdir(dir_path):
             ext = splitext(filename)[1].lower()
-            if ext in ('.c', '.cpp'):
+            if ext in C_FILE_EXTENSIONS:
                 self._parse_lines(os.path.join(dir_path, filename), source_file)
                 if source_file in self._c_files_map:
                     return
@@ -238,7 +257,7 @@ class CythonModuleTracer(FileTracer):
             return self._file_path_map[source_file]
         except KeyError:
             pass
-        abs_path = os.path.abspath(source_file)
+        abs_path = _find_dep_file_path(filename, source_file)
 
         if self.py_file and source_file[-3:].lower() == '.py':
             # always let coverage.py handle this case itself
